deferred class  AGUI_DRAG_AND_DROP_MIXIN

--
-- Every drag and drop aware component must act as a drag source
-- and as a drop target. 
--
-- The drag and drop aware widgets in the base distribution are
--
--    AGUI_LIST
--    AGUI_MULTICOLUMN_LIST
--    AGUI_TREE
--    AGUI_TEXT 
--    AGUI_TEXT_FIELD
--    AGUI_CLIENT_CONTROL
--    AGUI_SCROLLABLE_CLIENT_CONTROL
--   

   
feature {ANY}  -- drag and drop
   
   
   Drag_action_copy : INTEGER is 1
	 -- Copy the drag data
   
   Drag_action_move : INTEGER is 2
	 -- Move the drag data
   
   Drag_action_link : INTEGER is 3
	 -- Create a link to the dragged data
   
   
   Drag_state_no_action : INTEGER is 0
	 -- No action is in progress.
   
   Drag_state_on_drag_begin : INTEGER is 1 
	 -- We just started the drag and drop action.
   
   Drag_state_on_drop_enter : INTEGER is 2
	 -- The drop is inside the widget now.
   
   
   dnd_window : AGUI_WINDOW
	 -- Window to which the mixin belongs 
   
   state : INTEGER
	 -- State of the current drag and drop action.
	 -- We keep track of this, because some calls are state dependent
   
   
feature {ANY}   -- source operations
   
   
   set_default_drag_cursor(copy, move, link: AGUI_CURSOR) is
	 -- Sets the cursors that are used during this drag&drop 
	 -- action. This method should only called as a reaction of a  
	 -- 'on_drag_begin' event and is valid until the current drag ends.
      require
	 not mixin_window.is_zombie
	 state = Drag_state_on_drag_begin
      do
      end
   
   
   on_drag_begin : FUNCTION[ANY,TUPLE[INTEGER,INTEGER], TUPLE[ARRAY[STRING],BIT Integer_bits]
	 -- Called with (x,y) at the beginning of a drag. It returns
	 -- an arry of formats that the initiator can send and the
	 -- supported drag actions.
      
   set_on_drag_begin (action: FUNCTION[ANY,TUPLE[INTEGER,INTEGER], TUPLE[ARRAY[STRING],BIT Integer_bits]]) is
	 -- Set the 'on_drag_begin' action.
      require
	 not mixin_window.is_zombie
      do
	 on_drag_begin := action
      end
      
   
   on_drag_end : ROUTINE[ANY,TUPLE[INTEGER,INTEGER,INTEGER]]
	 -- Called with (x,y,action) at the end of drop. The initiator can
	 -- now destroy all data that is held during the drag.
      
   set_on_drag_end (action: ROUTINE[ANY,TUPLE[INTEGER,INTEGER,INTEGER]]) is
	 -- Set the 'on_drag_end' action.
      require
	 not mixin_window.is_zombie
      do
	 on_drag_end := action
      end
   
   
   on_drag_data : FUNCTION[ANY,TUPLE[STRING],STRING]
	 -- Called whenever the drag initiator should supply data.
	 -- The argument 'STRING' is the data format.
   
   set_on_drag_data (action: FUNCTION[ANY,TUPLE[STRING],STRING]) is
	 -- Set the 'on_drag_data' action.
      require
	 not mixin_window.is_zombie
      do
	 on_drag_data := action
      end
   
   
   
feature {ANY}  --  target operations

   
   set_default_drop_cursor(copy, move, link: AGUI_CURSOR) is
	 -- Sets the cursors that are used during this drag&drop 
	 -- action in this window. This method should only called as
	 -- a reaction of a 'on_drop_enter' event and is valid 
	 -- until the mouse leaves the window.
      require
	 not mixin_window.is_zombie
	 state = Drag_state_on_drop_enter
      do
	 efox_drag_and_drop_mixin__set_default_drop_cursor(dnd_handle,copy.handle,move.handle,link.handle)
      end
   
   
   set_drop_target(flag: BOOLEAN)is
	 -- Registers or unregisters the window as a target for drop actions.
	 -- This must be set before the window can receive any drop 
	 -- relevant messages.
      require
	 not mixin_window.is_zombie
      do
	 efox_drag_and_drop_mixin__set_drop_target(dnd_handle, flag)
      end
   
   
   query_drag_type (format: STRING): BOOLEAN is
	 -- Queries if the dragged data is in the given 'format'.
      require
	 not mixin_window.is_zombie
	 format /= Void
	 state > Drag_state_on_drag_begin
      do
	 result := efox_drag_and_drop_mixin__query_drag_type(dnd_handle, format.to_external)
      end
   
   
   retrieve_drag_data (format: STRING) : STRING is
	 -- Gets the dragged data in the requested data 'format'.
      require
	 not mixin_window.is_zombie
	 format /= Void and then query_drag_type(format)
	 state > Drag_state_on_drag_begin
      do
	 result := efox_drag_and_drop_mixin__retrieve_drag_data(dnd_handle, format.to_external)
      ensure
	 result /= Void
      end
   
   
   
   on_drop_enter : FUNCTION[ANY,TUPLE[INTEGER,INTEGER,BIT Integer_bits,INTEGER],INTEGER]
	 -- Called with (x,y, possible_actions, wanted_action) 
	 -- whenever the drag mouse cursor enters the widget.
	 -- Agent should return with the most wanted action.
   
   set_on_drop_enter (action: FUNCTION[ANY,TUPLE[INTEGER,INTEGER,BIT Integer_bits,INTEGER],INTEGER]) is
      require
	 not mixin_window.is_zombie
      do
	 on_drop_enter := action
      end
   
   
   on_drop_leave : FUNCTION[ANY,TUPLE[INTEGER,INTEGER,BIT Integer_bits,INTEGER],INTEGER]
	 -- Called with (x,y, possible_actions, wanted_action) 
	 -- whenever the drag mouse cursor left the widget.
	 -- Agent should return with the most wanted action.

   set_on_drop_leave (action: FUNCTION[ANY,TUPLE[INTEGER,INTEGER,BIT Integer_bits,INTEGER],INTEGER]) is
      require
	 not mixin_window.is_zombie
      do
	 on_drop_leave := action
      end
   
   
   on_drop_motion : FUNCTION[ANY,TUPLE[INTEGER,INTEGER,BIT Integer_bits,INTEGER],INTEGER]
	 -- Called with (x,y, possible_actions, wanted_action)
	 -- whenever the drag mouse cursor is moved inside the widget.
	 -- Agent should return with the most wanted action.
   
   set_on_drop_motion (action: FUNCTION[ANY,TUPLE[INTEGER,INTEGER,BIT Integer_bits,INTEGER],INTEGER]) is
      require
	 not mixin_window.is_zombie
      do
	 on_drop_motion := action
      end
   
   
   on_drop : ROUTINE[ANY,TUPLE[INTEGER,INTEGER,INTEGER]]
	 -- Called with (x,y,action) whenever the drop takes place. 
	 -- Now the data can be get via 'retrieve_drag_data'
   
   set_on_drop (action: ROUTINE[ANY,TUPLE[INTEGER,INTEGER,INTEGER]]) is
      require
	 not mixin_window.is_zombie
      do
	 on_drop := action
      end
   
   
feature {ANY} -- mixed inside
   
   
   mixin_window : AGUI_WINDOW is
	 -- Window in that this class is mixed in.
      deferred
      end
   
   
feature {NONE}  -- private C binding
   
   
   efox_drag_and_drop_mixin__retrieve_drag_data(hnd,ptr: POINTER) is
      external "C"
      end
   
   efox_drag_and_drop_mixin__retrieve_drag_data(hnd,ptr: POINTER) is
      external "C"
      end
   
   efox_drag_and_drop_mixin__set_default_drop_cursor(hnd,ch,mh,lh: POINTER) is
      external "C"
      end
   
   efox_drag_and_drop_mixin__set_drop_target(hnd: POINTER; flag: BOOLEAN) is
      external "C"
      end
   
   efox_drag_and_drop_mixin__set_default_drag_cursor(hnd,ch,mh,lh: POINTER) is
      external "C"
      end

end
