class AGUI_FONT_MANAGER
   
creation
   {AGUI_APPLICATION} make
   

feature {ANY}
   
   
   contains (name: STRING) : BOOLEAN is
	 -- Test if a font with a given logical name is
	 -- already registered.
      require
	 name /= Void
	 name.count > 0
      do
	 result := dict.has(name)
      end
   
   
   find (name: STRING) : AGUI_FONT is
	 -- Find a font by the given logical name. 
      require
	 name /= Void
	 name.count > 0
      do
	 if dict.has(name) then 
	    result := dict.at(name)
	 end
      end
   
   
   list_all_fonts : ARRAY[AGUI_FONT] is
	 -- Returns an array with all registered fonts.
	 -- The array is newly created on every call.
	 -- None of the fonts is in a zombie state.
      do
	 create result.with_capacity(dict.count,1)
	 dict.item_map_in(result)
      ensure
	 result /= Void and then result.lower = 1
      end
   
   
   list_all_names : ARRAY[STRING] is
	 -- Returns an array with the logical name of all registered fonts.
	 -- The array and all strings are newly created on every call.
	 -- None of the fonts is in a zombie state.
      local
	 i : INTEGER
      do
	 create result.with_capacity(dict.count,1)
	 dict.key_map_in(result)
	 from
	    i := result.lower
	 until
	    i > result.upper
	 loop
	    result.put(clone(result.item(i)),i)
	    i := i + 1
	 end
      ensure
	 result /= Void and then result.lower = 1
      end
   
   
   
   register (name, font_face: STRING; font_size: INTEGER; font_bold, font_italic: BOOLEAN) is
	 -- Register a new font with the logical 'name' for further 
	 -- use. The font properties face-name, font-size, font-bold, 
	 -- font-italic describe wich font should be used. If the 
	 -- font is not found, nothing is registered.
	 -- The size must be given in pixels.
      require
	 name /= Void and then name.count > 0      
	 font_face /= Void and then font_face.count > 0
	 font_size > 0	 
      local
	 font : AGUI_FONT
	 hnd : POINTER
	 n : STRING
      do
	 if not contains(name) then
	    last_register_status := false
	    hnd := efox_font_manager__register(font_face.to_external,font_size, font_bold,font_italic)
	    if hnd /= Void then
	       create font.make(hnd)
	       n := clone(name)
	       dict.put(font,n)
	       efox_font_manager__attach_name_and_object(hnd, n, font)
	       last_register_status := true
	    end
	 end
      end
   
   
   last_register_status : BOOLEAN
	 --  success status of last register operation
   
   
feature {AGUI_APPLICATION}
   
   
   register_application_fonts is
	 -- register the application fonts during startup
      external "C"
      alias "efox_font_manager__register_application_fonts"
      end
   
      
feature {NONE}  --  private implementation issues
   
   
   dict : DICTIONARY[AGUI_FONT, STRING]
	 -- Hashtable with all living registered icons
   
   zombie_dict : DICTIONARY[AGUI_FONT, STRING]
	 -- Hashtable with fonts icons that are still referenced 
	 -- from the C++ side of FOX.
   
   
   register_callback (name_ptr: POINTER; hnd: POINTER) is
	 -- Called from the C Side whenever a new font should be 
	 -- registered. We get the FOX handle and must create
	 -- a corresponding Eiffel AGUI_FONT object with the given name.
	 -- This callback is used when a FOX object creates a font - not
	 -- when an eiffel object does this.
      local
	 name: STRING; 
	 font : AGUI_FONT
      do
	 create name.from_external_copy(name_ptr)
	 create font.make(hnd)
	 dict.put(font, name)
	 efox_font_manager__attach_name_and_object(hnd, name, font)
      end
   
   
   unregister (font: AGUI_FONT; name: STRING; living: BOOLEAN) is
	 -- Called from the C Side whenever a font is marked as destroyed.
	 -- The parameter 'living' is true when the font is still referenced
	 -- from a widget.
      do
	 dict.remove(name)
	 if living then
	    zombie_dict.put(font,name)
	 end
      end
   
   
   unregister_zombie (name: STRING) is
	 -- Called from the C Side whenever a zombie font is really destroyed.
      do
	 zombie_dict.remove(name)
      end
   
   
feature {NONE}
   
   
   make is 
      do
	 create dict.with_capacity(20)
	 create zombie_dict.with_capacity(12)
	 efox_font_manager__make(Current)
      end
   
   
feature {NONE}
   
   
   efox_font_manager__make(this:like Current) is
	 -- initialize c side
      external "C"
      end
   
   efox_font_manager__register(face: POINTER; s: INTEGER; b,i: BOOLEAN) : POINTER is
	 -- register a new font
      external "C" 
      end
   
   efox_font_manager__attach_name_and_object(hnd: POINTER; n:STRING; f:AGUI_FONT) is
	 -- store name and eiffel object in the c side
      external "C"
      end
   
end
   
