class  AGUI_PROCESS
--
-- This class is used to start another process asynchron.
-- It can capture the standard output and standard error streams and
-- feed data to the process via the standard input stream.
-- 
   
inherit 
   MEMORY
      redefine dispose
      end
   
creation
   make
   

feature {ANY}
      
   
   Capture_none : INTEGER is 0
	 -- Capture nothing
   
   Capture_input : INTEGER is 1
      -- Capture only standard input
   
   Capture_output : INTEGER is 2
      -- Capture only standard output
   
   Capture_input_output : INTEGER is 3
      -- Capture input and output
   
   Capture_input_outputerror : INTEGER is 4
      -- Capture input and merged output+error
   
   Capture_input_output_error : INTEGER is 5
      -- Capture all
    
   
   capture_mode : INTEGER is
	 -- Return the capture mode.
      do
	 result := efox_process__capture_mode(handle)
      end
   
   set_capture_mode (mode: INTEGER) is
	 -- Set the capture mode for the next invocation of 'execute'
      require
	 mode.in_range(Capture_none, Capture_input_output_error)
      do
	 efox_process__set_capture_mode(handle,mode)
      end
   
   
feature {ANY}
   
   
   output : STRING
	 -- Keeps the accumulated output text
   
   error : STRING 
	 -- Keeps the accumulated error text
   
   
   feed_input (data: STRING) is
	 -- Feed the 'data' in string to the process
      require
	 data /= Void
      do
	 efox_process__feed_input(handle, data.to_external, data.count)
      end
   
   
   set_output_agent(action: ROUTINE[ANY,TUPLE[INTEGER]]) is
	 -- Sets the method that will be called whenever new output data
	 -- is available.  The argument is the number of bytes 
	 -- that is written to the output. 
	 -- The bytes are already appended to 'output'.
      do
	 output_agent := action
      end
   
   
   set_error_agent(action: ROUTINE[ANY,TUPLE[INTEGER]]) is
	 -- Sets the method that will be called whenever new error data
	 -- is available. The argument is the number of bytes 
	 -- that is written to the error output. 
	 -- The bytes are already appended to 'error'.
      do
	 error_agent := action
      end
   
   
   set_terminate_agent (action: ROUTINE[ANY,TUPLE]) is
      do
	 terminate_agent := action
      end
   
   
feature {ANY}
   
   
   execute (command: STRING) is
	 -- Execute the 'command', which can be everything you can
	 -- type on the command line. Because it will also execute a shell.
      require
	 command /= Void
	 not is_running
      do
	 efox_process__execute(handle,command.to_external)
      end
   
   is_running : BOOLEAN is
	 -- Test if the process is running
      do
	 result := efox_process__is_running(handle)
      end
   
   terminate (code: INTEGER) is
	 -- Terminate the process immediately.
      do
	 efox_process__terminate(handle,code)
      end
   
   
feature {NONE} -- constructor/destructor
   
   
   make is
      do
	 create output.make(2048)
	 create error.make(2048)
	 handle := efox_process__make(Current)
      end
   
   dispose is
      do
	 efox_process__dispose(handle)
      end
   
   
feature {NONE} -- private C binding
   
   
   handle : POINTER
   error_agent : ROUTINE[ANY,TUPLE[INTEGER]]
   output_agent : ROUTINE[ANY,TUPLE[INTEGER]]
   terminate_agent : ROUTINE[ANY,TUPLE]
   
   
   efox__callback_error (ptr: POINTER) is
      local
	 temp : STRING
      do
	 create temp.from_external_copy(ptr)
	 error.append_string(temp)
	 if error_agent /= Void then error_agent.call([temp.count]) end
      end

   efox__callback_output (ptr: POINTER) is
      local
	 temp : STRING
      do
	 create temp.from_external_copy(ptr)
	 output.append_string(temp);
	 if output_agent /= Void then output_agent.call([temp.count]) end
      end

   efox__callback_terminate is
      do
	 if terminate_agent /= Void then terminate_agent.call([]) end
      end
   
   efox_process__make(this:like Current) : POINTER is
      external "C"
      end
      
   efox_process__dispose (hnd: POINTER) is
      external "C"
      end
   
   efox_process__set_capture_mode(hnd: POINTER; n: INTEGER) is
      external "C"
      end
   
   efox_process__capture_mode(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_process__execute(hnd: POINTER; cmd: POINTER) is
      external "C"
      end
   
   efox_process__is_running(hnd: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_process__terminate(hnd: POINTER; n: INTEGER) is
      external "C"
      end      
   
   efox_process__feed_input(hnd: POINTER; s: POINTER; l: INTEGER) is
      external "C"
      end
   
end
