class  AGUI_REGULAR_EXPRESSION
   
inherit
   MEMORY
      redefine dispose
      end
   
   
creation
   make
   
   
feature {ANY}  -- compiling a pattern
   
   
   compile (pattern: STRING; case_insensitive: BOOLEAN) is
	 -- Compile a new regular expression.
	 -- With the parameter 'case_insensitive' you can force
	 -- that the pattern mathc both upper and lower case.
      require
	 pattern /= Void
	 pattern.occurrences('%U') = 0
      do
	 last_compile_error := Void
	 if handle /= default_pointer then
	    efox_regular_expression__free(handle)
	 end
	 handle := efox_regular_expression__compile(pattern.to_external,$last_compile_error_offset,$temp_ptr,case_insensitive)
	 if handle = default_pointer then
	    create last_compile_error.from_external_copy(temp_ptr)
	 end
      end
   
   
   compile_multiline (pattern: STRING; case_insensitive: BOOLEAN) is
	 -- Compile a new regular expression that handles embedded
	 -- newline characters in the subject string as points that
	 -- match the '^' and '$' metacharacters.
	 -- With the parameter 'case_insensitive' you can force
	 -- that the pattern mathc both upper and lower case.
      require
	 pattern /= Void
	 pattern.count > 0
	 pattern.occurrences('%U') = 0	 	 
      do
	 last_compile_error := Void
	 if handle /= default_pointer then
	    efox_regular_expression__free(handle)
	 end
	 handle := efox_regular_expression__compile_multiline(pattern.to_external,$last_compile_error_offset,$temp_ptr,case_insensitive)
	 if handle = default_pointer then
	    create last_compile_error.from_external_copy(temp_ptr)
	 end
      end
   
   
   
   is_compiled : BOOLEAN is
	 -- Test if we have a compiled regular expression.
      do
	 result := handle /= default_pointer
      end
   
   
   last_compile_error : STRING
	 -- If the last 'compile' failed, this contains a human readable
	 -- textual error message.
   
   
   last_compile_error_offset : INTEGER
	 -- Offset in pattern where the last error during 'compile' occurred.
   
   
   optimize is
	 -- Try to optimize the compiled pattern for faster matching.
	 -- This is only usefull if the pattern is used a lot.
      require
	 is_compiled
      do 
	 efox_regular_expression__optimize(handle)
      end
   
   
   free is
	 -- Free the memory for the last compiled expression.
      do
	 efox_regular_expression__free(handle)
	 handle := default_pointer
      end
   
   
feature {ANY} -- matching a pattern
   
   
   match (str: STRING) is
	 -- Test if the regular expression matches the string beginning
	 -- at offset 1.
      require
	 not_void: str /= Void
	 must_be_compiled: is_compiled
      do
	 match_start_offset := 0
	 match_end_offset := 0
	 efox_regular_expression__match(handle, str.to_external, str.count,$count_captured_substrings)
	 if count_captured_substrings > 0 then
	    efox_regular_expression__captured_substring(handle, 1, $match_start_offset, $match_end_offset)
	 end
      end
   
   
   search (str: STRING) is
	 -- Search for the first substring that matches the regular
	 -- expression.
      require
	 not_void: str /= Void
	 must_be_compiled: is_compiled
      do	 
	 match_start_offset := 0
	 match_end_offset := 0
	 efox_regular_expression__search(handle, str.to_external, str.count,$count_captured_substrings)
	 if count_captured_substrings > 0 then
	    efox_regular_expression__captured_substring(handle, 1, $match_start_offset, $match_end_offset)
	 end
      end
   
   
   search_substring(str: STRING; start_index: INTEGER) is
	 -- Search for the first substring not before 'start_index'
	 -- that matches the regular expression. 
      require
	 not_void: str /= Void
	 valid_start_index: start_index >= 1 and start_index <= str.count + 1
	 must_be_compiled: is_compiled			    
      do
	 efox_regular_expression__search_substring(handle, str.to_external, str.count, start_index,$count_captured_substrings)
	 if count_captured_substrings > 0 then
	    efox_regular_expression__captured_substring(handle, 1, $match_start_offset, $match_end_offset)
	 end
      end
      
   
   count_captured_substrings : INTEGER
	 -- Return number of captured substrings during last matching
	 -- operation.
   
   
   captured_substring (idx: INTEGER; data: TUPLE[INTEGER,INTEGER]) is
	 -- Return the offset values for the captured substring.
	 -- The first integer in the tuple is filled with the start
	 -- offset, the second one with the end offset.
	 -- The first substring is the complete match of the pattern.
	 -- The offset can also be negative (-1) if the corresponding substring
	 -- does not exist. This is necessary to differentiate it from
	 -- the empty string.
      require
  	 idx.in_range(1, count_captured_substrings)
	 is_compiled
      do
	 efox_regular_expression__captured_substring(handle, idx, $temp_begin, $temp_end)
	 if temp_end = -1 or else temp_begin = - 1 then
	    data.set_first(-1)
	    data.set_second(-1)
	 else
	    data.set_first(temp_begin)
	    data.set_second(temp_end)
	 end
      end
   
   
   substring (n: INTEGER; s: STRING) : STRING is
	 -- Retrieve the n'th captured substring. Return 'Void' if
	 -- there is no substring. 's' must be the previous searched
	 -- or matched string !
      require
	 is_compiled
      local
	 t: TUPLE[INTEGER,INTEGER]
      do
  	 if n.in_range(1, count_captured_substrings) then
	    create t
	    captured_substring(n, t)
	    if t.first /= -1 then
	       check t.second /= -1 end
	       check s.valid_index(t.first) end
	       check s.valid_index(t.second) end
	       result := s.substring(t.first,t.second)
	    end
	 end
      end
   
   
   match_start_offset : INTEGER
	 -- Start offset of the substring that matches the entire 
	 -- regular expression. Invalid if count_captured_substring 
	 -- is 0.
   
   
   match_end_offset : INTEGER
	 -- End offset of the substring that matches the entire 
	 -- regular expression. Invalid if count_captured_substring 
	 -- is 0.
      

feature {NONE} -- creation
   
   
   make is
      do
      end
   
   
feature {NONE} -- private definitions
   
   
   handle : POINTER
	 -- handle to the PCRE pattern
   
   temp_ptr : POINTER
	 -- used to return the error message from C
   
   temp_begin, temp_end: INTEGER
	 -- temporary storage area when accessing captures substrings
   
   dispose is
	 -- free all allocated memory
      do
	 if handle /= default_pointer then
	    free
	 end
      end
   
   
feature {NONE} -- private C binding
   
   
   efox_regular_expression__compile(p,op,ep: POINTER; icase: BOOLEAN) : POINTER is
      external "C"
      end
   
   efox_regular_expression__compile_multiline(p,op,ep: POINTER; icase: BOOLEAN) : POINTER is
      external "C"
      end
   
   efox_regular_expression__free(p: POINTER) is
      external "C"
      end
   
   efox_regular_expression__match(hnd,str: POINTER; len: INTEGER; countptr: POINTER) is
      external "C"
      end
      
   efox_regular_expression__search(hnd,str: POINTER; len: INTEGER; countptr: POINTER) is
      external "C"
      end
   
   efox_regular_expression__search_substring(hnd,str: POINTER; len,start_index: INTEGER; countptr: POINTER) is
      external "C"
      end
	 
   efox_regular_expression__captured_substring(hnd: POINTER; idx: INTEGER; pbegin,pend: POINTER) is
      external "C"
      end
   
   efox_regular_expression__optimize(hnd: POINTER) is
      external "C"
      end
   
end
