--
--  Splitter window is used to interactively repartition
--  two or more subpanes.
--  Space may be subdivided horizontally or vertically.
--  When the splitter is itself resized, the right-most (bottom-most)
--  child window will be resized unless the splitter window is reversed;
--  if the splitter is reversed, the left-most (top-most) child window
--  will be resized instead.
--  The splitter widget sends a SEL_CHANGED to its target
--  during the resizing of the panes; at the end of the resize interaction,
--  it sends a SEL_COMMAND to signify that the resize operation is complete.
--  Normally, children are resizable from 0 upwards; however, if the child
--  in a horizontally oriented splitter has LAYOUT_FILL_X in combination with 
--  LAYOUT_FIX_WIDTH, it will not be made smaller than its default width,
--  except when the child is the last visible widget (or first when the option
--  SPLITTER_REVERSED has been passed to the splitter).
--  In a vertically oriented splitter, children with LAYOUT_FILL_Y and 
--  LAYOUT_FIX_HEIGHT behave analogously.
--  These options only affect interactive resizing.
--

class  AGUI_SPLITTER
      
   
inherit
   AGUI_COMPOSITE
      end
   
   
creation
   {ANY} make
   
   
feature {NONE} -- constructor

   
   make (p: AGUI_COMPOSITE;  n: STRING) is
      require
	 p /= Void and n /= Void
      do
	 handle := efox_splitter__make(Current,p.handle, n.to_external, $temp_px)
	 efox__window_hash_table.put(Current, temp_px)
      end


feature {ANY} -- public interface   

   
   bar_size : INTEGER is
	 -- Return size of the splitter bar in pixel.
      require
	 not is_zombie
      do
	 result := efox_splitter__bar_size(handle)
      end


   set_bar_size (n: INTEGER) is
	 -- Sets the width of the splitter bar to 'n' pixel.
      require
	 not is_zombie
	 n >= 0
      do
	 efox_splitter__set_bar_size(handle,n)
      end   
   
   
   horizontal_orientation : BOOLEAN is
	 -- Test if splitter has horizontal or vertical orientation. 
      require
	 not is_zombie
      do
	 result := efox_splitter__horizontal_orientation(handle)
      end
   
   
   set_horizontal_orientation (flag: BOOLEAN) is
	 -- Set if splitter has horizontal or vertical orientation. 
      require
	 not is_zombie
      do
	 efox_splitter__set_horizontal_orientation(handle,flag)
      end
   
   
   tracking : BOOLEAN is
	 -- Test if the child windows are resized while dragging 
	 -- the splitter bar.
      require
	 not is_zombie
      do
	 result := efox_splitter__tracking(handle)
      end
   
   
   set_tracking (flag: BOOLEAN) is
	 -- Set if the child windows should be resized while dragging 
	 -- the splitter bar.
      require
	 not is_zombie
      do
	 efox_splitter__set_tracking(handle,flag)
      end
   
   
   reversed : BOOLEAN is
	 -- Test if the splitter is in reversed mode. In this mode the
	 -- first child window will be resized, otherwise the last 
	 -- will be resized.
      require
	 not is_zombie
      do
	 result := efox_splitter__reversed(handle)
      end
   
   
   set_reversed (flag: BOOLEAN) is
	 -- Set the splitter in reversed mode. In this mode the
	 -- first child window will be resized, otherwise the last 
	 -- will be resized.
      do
	 efox_splitter__set_reversed(handle,flag)
      end
   
   
feature {NONE} -- private C binding
   
   
   efox_splitter__make(this:like Current; hnd,n,key: POINTER) : POINTER is
      external "C"
      end
   
   efox_splitter__bar_size(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_splitter__set_bar_size(hnd: POINTER; n: INTEGER) is
      external "C"
      end
   
   efox_splitter__horizontal_orientation(hnd: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_splitter__set_horizontal_orientation(hnd: POINTER; flag: BOOLEAN) is
      external "C"
      end
   
   efox_splitter__tracking(hnd: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_splitter__set_tracking(hnd: POINTER; flag: BOOLEAN) is
      external "C"
      end
   
   efox_splitter__reversed(hnd: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_splitter__set_reversed(hnd: POINTER; flag: BOOLEAN) is
      external "C"
      end
   
   
end
   
